@@ -6,29 +6,70 @@ use POSIX;
 use List::Util;
 use Data::Dumper qw(Dumper);
 use Exporter qw(import);
+our $VERSION = '0.41';
 our @EXPORT_OK = qw(
-    incr        reduces  flatten
-    drop_right  drop     take_right  take
-    assoc       maps     decr        chain
-    first       end      subarray    partial
-    __          find     filter      some
-    none        uniq     bool        spread
-    len         to_keys  to_vals     is_array
-    is_hash     every    noop        identity
-    is_empty    is_sub   flow        eql
-    to_pairs    for_each apply       get
-    second      range    pops        pushes
-    shifts      unshifts once
+    incr        reduces   flatten
+    drop_right  drop      take_right  take
+    assoc       maps      decr        chain
+    first       end       subarray    partial
+    __          find      filter      some
+    none        uniq      bool        spread
+    len         to_keys   to_vals     is_array
+    is_hash     every     noop        identity
+    is_empty    is_sub    flow        eql
+    to_pairs    for_each  apply       get
+    second      range     pops        pushes
+    shifts      unshifts  once
 );
-
-our $VERSION = '0.31';
-
 use constant ARG_PLACE_HOLDER => {};
 
+_wrap_to_use_partials(
+    grep { $_ !~ /flow|flow_right|partial|chain/ } @EXPORT_OK
+);
+
 # -----------------------------------------------------------------------------#
 
 sub __ { ARG_PLACE_HOLDER };
 
+sub _contains_placeholders {
+    my $arguments = [@_];
+
+    foreach my $arg (@{ $arguments }) {
+        if (ref($arg) and ref(__) and $arg == __) {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+sub _wrap {
+    no warnings 'redefine';
+    no strict 'refs';
+    my ($methodName, $decorator) = @_;
+    my $module                    = caller;
+    my $fullName                  = "$module"."::"."$methodName";
+    my $wrappedSub                = *{"$fullName"}{CODE};
+
+    *{"$fullName"} = List::Util::reduce {
+       my ($wrappedSub, $decor) = ($a, $b);
+       return $decor->($wrappedSub);
+    } ($wrappedSub, $decorator);
+}
+
+sub _can_use_placeholders {
+    my $func = shift;
+
+    return sub {
+        return _contains_placeholders(@_) ? partial($func, @_) : $func->(@_);
+    }
+}
+
+sub _wrap_to_use_partials {
+    foreach my $func_name (@_) {
+        _wrap($func_name, \&_can_use_placeholders);
+    }
+}
+
 sub noop { return undef }
 
 sub identity {
@@ -107,9 +148,10 @@ sub _is_nonsense_range {
     }
 }
 
-sub get {
-    my $coll    = shift // [];
+
+sub _safe_get {
     my $key     = shift // 0;
+    my $coll    = shift // [];
     my $default = shift;
 
     if (is_array($coll)) {
@@ -125,6 +167,21 @@ sub get {
     return defined $string_coll->[$key] ? $string_coll->[$key] : $default;
 }
 
+sub get {
+    my ($path, $coll, $default) = @_;
+
+    if (!len($path)) {
+        return $default;
+    }
+
+    my $accessors = [$path =~ /([\w\s-]+)/g];
+
+    return reduces(sub {
+        my ($value, $accessor) = @_;
+        return _safe_get($accessor, $value, $default);
+    }, $coll, $accessors)
+}
+
 sub apply {
     my $fn   = shift // sub {};
     my $args = shift // [];
@@ -133,19 +190,19 @@ sub apply {
 }
 
 sub flow {
-    my $args = [@_];
+    my $funcs = [@_];
 
-    if (ref $args->[0] ne 'CODE') {
+    if (ref $funcs->[0] ne 'CODE') {
         return \&noop;
     }
 
     return sub {
-        my $seed = shift // \&noop;
+        my $args = [@_];
 
-        return List::Util::reduce {
-            my ($accum, $func) = ($a, $b);
-            $func->($accum);
-        } ($seed, spread($args));
+        return chain(
+            sub { first($funcs)->(spread($args)) },
+            spread(drop($funcs)),
+        );
     }
 }
 
@@ -252,6 +309,8 @@ sub for_each {
         $idx++;
         $fn->($val, $idx - 1, $coll);
     }
+
+    return undef;
 }
 
 sub is_empty {
@@ -456,6 +515,7 @@ sub maps {
     return [@vals];
 }
 
+
 sub reduces {
     my $func           = shift;
     my ($accum, $coll) = spread(_get_reduces_args([@_]));
@@ -495,8 +555,9 @@ sub partial {
 
     return sub {
         my $newArgs = [@_];
-        my $no_placeholder_args = _fill_holders($oldArgs, $newArgs);
-        return $func->(@$no_placeholder_args);
+        my $filled_args = _fill_holders([spread($oldArgs)], $newArgs);
+
+        return $func->(spread($filled_args));
     }
 }
 
@@ -511,7 +572,7 @@ sub _fill_holders {
     for (my $idx = 0; $idx < $old_args_len; $idx++) {
         my $arg = shift @{ $old_args };
 
-        if (eql($arg, __)) {
+        if(_contains_placeholders($arg)) {
             push @{ $filled_args }, (shift @{ $new_args });
         } else {
             push @{ $filled_args }, $arg;
@@ -589,6 +650,8 @@ sub spread {
     return split('', $coll);
 }
 
+# ------------------------------------------------------------------------------
+
 =head1 NAME
 
 Sub::Fp - A Clojure / Python Toolz / Lodash inspired Functional Utility Library
@@ -1133,21 +1196,21 @@ Only works one level deep;
         key1 => 'value1',
     };
 
-    get($hash, 'key1');
+    get('key1', $hash);
 
     # 'value1'
 
 
     my $array = [100, 200, 300]
 
-    get($array, 1);
+    get(1, $array);
 
     # 200
 
 
     my $string = "Hello";
 
-    get($string, 1);
+    get(1, $string);
 
     # e
 
@@ -1158,7 +1221,7 @@ Only works one level deep;
         key1 => 'value1',
     };
 
-    get($hash, 'key2', "DEFAULT HERE");
+    get('key2', $hash, "DEFAULT HERE");
 
     # 'DEFAULT HERE'
 
